/*********************************************************************
*                 SEGGER Software GmbH                               *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2018  SEGGER Microcontroller GmbH                *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.48 - Graphical user interface for embedded applications **
All  Intellectual Property rights in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product. This file may
only be used in accordance with the following terms:

The  software has  been licensed by SEGGER Software GmbH to Nuvoton Technology Corporationat the address: No. 4, Creation Rd. III, Hsinchu Science Park, Taiwan
for the purposes  of  creating  libraries  for its 
Arm Cortex-M and  Arm9 32-bit microcontrollers, commercialized and distributed by Nuvoton Technology Corporation
under  the terms and conditions  of  an  End  User  
License  Agreement  supplied  with  the libraries.
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
Licensing information
Licensor:                 SEGGER Software GmbH
Licensed to:              Nuvoton Technology Corporation, No. 4, Creation Rd. III, Hsinchu Science Park, 30077 Hsinchu City, Taiwan
Licensed SEGGER software: emWin
License number:           GUI-00735
License model:            emWin License Agreement, signed February 27, 2018
Licensed platform:        Cortex-M and ARM9 32-bit series microcontroller designed and manufactured by or for Nuvoton Technology Corporation
----------------------------------------------------------------------
Support and Update Agreement (SUA)
SUA period:               2018-03-26 - 2019-03-27
Contact to extend SUA:    sales@segger.com
----------------------------------------------------------------------
File        : 2DGL_DrawPNG.c
Purpose     : Example for drawing PNG images
Requirements: WindowManager - ( )
              MemoryDevices - ( )
              AntiAliasing  - ( )
              VNC-Server    - ( )
              PNG-Library   - (x)
              TrueTypeFonts - ( )

              Can be used in a MS Windows environment only!
----------------------------------------------------------------------
*/
#include "ff.h"
#include "GUI.h"
#include "WM.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
/*********************************************************************
*
*       Static data
*
**********************************************************************
*/
#ifdef __USE_SD__
#else
static unsigned char _acKoalaPNG[] = {
    #include "Koala.c"
};

static unsigned char _acTulipsPNG[] = {
    #include "Tulips.c"
};

static unsigned char _acDesertPNG[] = {
    #include "Desert.c"
};

static unsigned char _acPenguinsPNG[] = {
    #include "Penguins.c"
};
#endif

extern volatile int end_flag;

#ifdef __USE_SD__
/*********************************************************************
*
*       _GetData
*/
static int _GetData(void * p, const U8 ** ppData, unsigned NumBytesReq, U32 Off) {
    FIL hFile1;
    UINT NumBytesRead;
    FRESULT res;
  U8     * pData;

  pData  = (U8 *)*ppData;
    hFile1 = *(FIL *)p;
  //
  // Set file pointer to the required position
  //
    f_lseek(&hFile1, Off);
  //
  // Read data into buffer
  //
    res = f_read(&hFile1, pData, NumBytesReq, &NumBytesRead);
    if ( res != FR_OK )
        sysprintf("### read fail\n");
  //
  // Return number of available bytes
  //
  return NumBytesRead;
}

/*********************************************************************
*
*       _DrawPNG_file
*
* Function description
*   Draws a PNG file from a file system
*/
static void _DrawPNG_file(const char * sFilename) {
    FIL hFile;
  int    xSize;
  int    ySize;
  int    w;
  int    h;
  int    xPos;
  int    yPos;

//  hFile = CreateFile(sFilename, GENERIC_READ, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
    f_open(&hFile, sFilename, FA_OPEN_EXISTING | FA_READ);
  w     = GUI_PNG_GetXSizeEx(_GetData, &hFile);
  h     = GUI_PNG_GetYSizeEx(_GetData, &hFile);
  xSize = LCD_GetXSize();
  ySize = LCD_GetYSize();
  xPos  = (xSize - w) / 2 + 10;
  yPos  = (ySize - h) / 2 + 10;
    end_flag = 0;
    GUI_MULTIBUF_Begin();
  GUI_PNG_DrawEx(_GetData, &hFile, xPos, yPos);
    GUI_MULTIBUF_End();
    while ( end_flag == 0 );
//  CloseHandle(hFile);
    f_close(&hFile);
}
#else
/*********************************************************************
*
*       _DrawPNG_mem
*
* Function description
*   Draws a PNG file from memory
*/
static void _DrawPNG_mem(const void * pFileData, int FileSize) {
  int xSize;
  int ySize;
  int w;
  int h;
  int xPos;
  int yPos;

  w     = GUI_PNG_GetXSize(pFileData, FileSize);
  h     = GUI_PNG_GetYSize(pFileData, FileSize);
  xSize = LCD_GetXSize();
  ySize = LCD_GetYSize();
  xPos  = (xSize - w) / 2 - 10;
  yPos  = (ySize - h) / 2 - 10;
    end_flag = 0;
    GUI_MULTIBUF_Begin();
  GUI_PNG_Draw(pFileData, FileSize, xPos, yPos);
    GUI_MULTIBUF_End();
    while ( end_flag == 0 );
}
#endif
/*********************************************************************
*
*       Public code
*
**********************************************************************
*/
/*********************************************************************
*
*       MainTask
*/
void MainTask(void) {
  GUI_Init();
    WM_MULTIBUF_Enable(1);

  while (1) {
#ifdef __USE_SD__ 
        _DrawPNG_file("0:\\Koala.png"); /* Please change the link to a valid PNG file */
        GUI_Delay(1000);
      _DrawPNG_file("0:\\Tulips.png");
    GUI_Delay(1000);
      _DrawPNG_file("0:\\Desert.png");
      GUI_Delay(1000);
      _DrawPNG_file("0:\\Penguins.png");
      GUI_Delay(1000);
#else
      _DrawPNG_mem(_acKoalaPNG, sizeof(_acKoalaPNG));
    GUI_Delay(1000);
      _DrawPNG_mem(_acTulipsPNG, sizeof(_acTulipsPNG));
    GUI_Delay(1000);
      _DrawPNG_mem(_acDesertPNG, sizeof(_acDesertPNG));
      GUI_Delay(1000);
      _DrawPNG_mem(_acPenguinsPNG, sizeof(_acPenguinsPNG));
      GUI_Delay(1000);
#endif
  }
}

/*************************** End of file ****************************/
